@@ -1,4 +1,4 @@
-/* $Id: stun_msg.h 5987 2019-05-14 09:31:39Z nanang $ */
+/* $Id$ */
 /* 
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -308,6 +308,10 @@ typedef enum pj_stun_msg_type
      */
     PJ_STUN_CHANNEL_BIND_ERROR_RESPONSE	    = 0x0119,
 
+    /**
+     * STUN/TURN Connect Request
+     */
+    PJ_STUN_CONNECT_REQUEST                 = 0x000a,
 
     /**
      * STUN/TURN ConnectBind Request
