@@ -1,4 +1,4 @@
-/* $Id: endpoint.hpp 6026 2019-06-12 06:00:35Z nanang $ */
+/* $Id$ */
 /* 
  * Copyright (C) 2013 Teluu Inc. (http://www.teluu.com)
  *
@@ -130,6 +130,13 @@ struct SslCertName
 {
     pj_ssl_cert_name_type  type;    	    /**< Name type		*/
     string		   name;    	    /**< The name		*/
+
+public:
+    /**
+     * Default constructor
+     */
+    SslCertName() : type(PJ_SSL_CERT_NAME_UNKNOWN)
+    {}
 };
 
 /** Array of SSL certificate type and name. */
@@ -340,6 +347,7 @@ struct IpChangeParam {
      * Default : PJSUA_TRANSPORT_RESTART_DELAY_TIME
      */
     unsigned	    restartLisDelay;
+
 public:
     /**
      * Constructor.
@@ -429,6 +437,150 @@ struct OnMediaEventParam
     MediaEvent      ev;
 };
 
+/**
+ * This structure describes authentication challenge used in Proxy-Authenticate
+ * or WWW-Authenticate for digest authentication scheme.
+ */
+struct DigestChallenge
+{
+    /**
+     * Realm for the challenge.
+     * */
+    std::string realm;
+
+    /*
+     * Other parameters.
+     * */
+    StringToStringMap otherParam;
+
+    /**
+     * Domain.
+     */
+    std::string domain;
+
+    /**
+     * Nonce challenge.
+     */
+    std::string nonce;
+
+    /**
+     * Opaque value.
+     */
+    std::string opaque;
+
+    /**
+     * Stale parameter.
+     */
+    int  stale;
+
+    /**
+     * Algorithm parameter.
+     */
+    std::string algorithm;
+
+    /**
+     * Quality of protection.
+     */
+    std::string qop;
+
+    /**
+     * Convert from pjsip
+     */
+    void fromPj(const pjsip_digest_challenge &prm);
+
+    /**
+     * Convert to pjsip
+     */
+    pjsip_digest_challenge toPj() const;
+};
+
+/**
+ * This structure describe credential used in Authorization and
+ * Proxy-Authorization header for digest authentication scheme.
+ */
+struct DigestCredential
+{
+    /**
+     *Realm of the credential
+     */
+    std::string realm;
+
+    /**
+     *Other parameters.
+     */
+    StringToStringMap otherParam;
+
+    /**
+     *Username parameter.
+     */
+    std::string username;
+
+    /**
+     *Nonce parameter.
+     */
+    std::string nonce;
+
+    /**
+     *URI parameter.
+     */
+    std::string uri;
+
+    /**
+     *Response digest.
+     */
+    std::string response;
+
+    /**
+     *Algorithm.
+     */
+    std::string algorithm;
+
+    /**
+     *Cnonce.
+     */
+    std::string cnonce;
+
+    /**
+     *Opaque value.
+     */
+    std::string opaque;
+
+    /**
+     *Quality of protection.
+     */
+    std::string qop;
+
+    /**
+     *Nonce count.
+     */
+    std::string nc;
+
+    /**
+     * Convert from pjsip
+     */
+    void fromPj(const pjsip_digest_credential &prm);
+
+    /**
+     * Convert to pjsip
+     */
+    pjsip_digest_credential toPj() const;
+};
+
+
+/**
+ * Parameters for onCredAuth account method.
+ */
+struct OnCredAuthParam
+{
+    DigestChallenge digestChallenge;
+
+    AuthCredInfo credentialInfo;
+
+    std::string method;
+
+    DigestCredential digestCredential;
+};
+
 //////////////////////////////////////////////////////////////////////////////
 /**
  * SIP User Agent related settings.
@@ -799,7 +951,9 @@ public:
     unsigned		rxDropPct;
 
     /**
-     * Echo canceller options (see pjmedia_echo_create())
+     * Echo canceller options (see pjmedia_echo_create()).
+     * Specify PJMEDIA_ECHO_USE_SW_ECHO here if application wishes
+     * to use software echo canceller instead of device EC.
      *
      * Default: 0.
      */
@@ -856,6 +1010,14 @@ public:
      */
     int			jbMax;
 
+    /**
+     * Set the algorithm the jitter buffer uses to discard frames in order to
+     * adjust the latency.
+     *
+     * Default: PJMEDIA_JB_DISCARD_PROGRESSIVE
+     */
+    pjmedia_jb_discard_algo jbDiscardAlgo;
+
     /**
      * Specify idle time of sound device before it is automatically closed,
      * in seconds. Use value -1 to disable the auto-close feature of sound
@@ -1089,6 +1251,11 @@ public:
 
     /**
      * Write a log entry.
+     * Application must implement its own custom LogWriter and
+     * this function will then call the LogWriter::write() method.
+     * Note that this function does not call PJSIP's internal
+     * logging functionality. For that, you should use
+     * utilLogWrite(prmLevel, prmSender, prmMsg) above.
      *
      * @param e			The log entry.
      */
@@ -1380,6 +1547,7 @@ public:
      */
     unsigned mediaActivePorts() const;
 
+#if !DEPRECATED_FOR_TICKET_2232
     /**
      * Warning: deprecated, use mediaEnumPorts2() instead. This function is
      * not safe in multithreaded environment.
@@ -1389,6 +1557,7 @@ public:
      * @return		The list of media port.
      */
     const AudioMediaVector &mediaEnumPorts() const PJSUA2_THROW(Error);
+#endif
 
     /**
      * Enumerate all audio media port.
@@ -1422,6 +1591,7 @@ public:
      * Codec management operations
      */
 
+#if !DEPRECATED_FOR_TICKET_2232
     /**
      * Warning: deprecated, use codecEnum2() instead. This function is not
      * safe in multithreaded environment.
@@ -1431,6 +1601,7 @@ public:
      * @return		Array of codec info.
      */
     const CodecInfoVector &codecEnum() PJSUA2_THROW(Error);
+#endif
 
     /**
      * Enum all supported codecs in the system.
@@ -1473,6 +1644,7 @@ public:
     void codecSetParam(const string &codec_id,
 		       const CodecParam param) PJSUA2_THROW(Error);
 
+#if !DEPRECATED_FOR_TICKET_2232
     /**
      * Warning: deprecated, use videoCodecEnum2() instead. This function is
      * not safe in multithreaded environment.
@@ -1482,6 +1654,7 @@ public:
      * @return		Array of video codec info.
      */
     const CodecInfoVector &videoCodecEnum() PJSUA2_THROW(Error);
+#endif
 
     /**
      * Enum all supported video codecs in the system.
@@ -1533,6 +1706,23 @@ public:
      */
     void resetVideoCodecParam(const string &codec_id) PJSUA2_THROW(Error);
 
+#if defined(PJMEDIA_HAS_OPUS_CODEC) && (PJMEDIA_HAS_OPUS_CODEC!=0)
+    /**
+     * Get codec Opus config.
+     *
+     */
+     CodecOpusConfig getCodecOpusConfig() const PJSUA2_THROW(Error);
+
+    /**
+     * Set codec Opus config.
+     *
+     * @param opus_cfg	Codec Opus configuration.
+     *
+     */
+    void setCodecOpusConfig(const CodecOpusConfig &opus_cfg)
+			    PJSUA2_THROW(Error);
+#endif
+
     /**
      * Enumerate all SRTP crypto-suite names.
      *
@@ -1560,8 +1750,6 @@ public:
      *    (configurable via \a AccountConfig.ipChangeConfig.reinviteFlags).
      *
      * @param param	The IP change parameter, have a look at #IpChangeParam.
-     *
-     * @return		PJ_SUCCESS on success, other on error.
      */
     void handleIpChange(const IpChangeParam &param) PJSUA2_THROW(Error);
 
@@ -1650,17 +1838,54 @@ public:
     virtual void onMediaEvent(OnMediaEventParam &prm)
     { PJ_UNUSED_ARG(prm); }
 
+    /**
+     * Callback for computation of the digest credential.
+     *
+     * Usually, an application does not need to implement (overload) this callback.
+     * Use it, if your application needs to support Digest AKA authentication without 
+     * the default digest computation back-end (i.e: using <b>libmilenage</b>).
+     *
+     * To use Digest AKA authentication, add \a PJSIP_CRED_DATA_EXT_AKA flag in the
+     * AuthCredInfo's \a dataType field of the AccountConfig, and fill up other
+     * AKA specific information in AuthCredInfo:
+     *  - If PJSIP_HAS_DIGEST_AKA_AUTH is disabled, you have to overload this callback
+     *    to provide your own digest computation back-end.
+     *  - If PJSIP_HAS_DIGEST_AKA_AUTH is enabled, <b>libmilenage</b> library from 
+     *    \a third_party directory is linked, and this callback returns PJ_ENOTSUP,
+     *    then the default digest computation back-end is used.
+     *
+     * @param prm.digestChallenge	The authentication challenge sent by server in 401
+     *		    or 401 response, as either Proxy-Authenticate or
+     *		    WWW-Authenticate header.
+     * @param prm.credentialInfo	    The credential to be used.
+     * @param method    The request method.
+     * @param prm.digestCredential	    The digest credential where the digest response
+     *		    will be placed to. Upon calling this function, the
+     *		    nonce, nc, cnonce, qop, uri, and realm fields of
+     *		    this structure must have been set by caller. Upon
+     *		    return, the \a response field will be initialized
+     *		    by this function.
+     *
+     * @return PJ_ENOTSUP is the default. If you overload this callback,
+     *		    return PJ_SUCCESS on success. 
+     */
+    virtual pj_status_t onCredAuth(OnCredAuthParam &prm);
+
 private:
     static Endpoint		*instance_;	// static instance
     LogWriter			*writer;	// Custom writer, if any
-    AudioMediaVector 	 	 mediaList;
     AudDevManager		 audioDevMgr;
     VidDevManager		 videoDevMgr;
+#if !DEPRECATED_FOR_TICKET_2232
     CodecInfoVector		 codecInfoList;
     CodecInfoVector		 videoCodecInfoList;
+#endif
     std::map<pj_thread_t*, pj_thread_desc*> threadDescMap;
     pj_mutex_t			*threadDescMutex;
+#if !DEPRECATED_FOR_TICKET_2232
+    AudioMediaVector 	 	 mediaList;
     pj_mutex_t			*mediaListMutex;
+#endif
 
     /* Pending logging */
     bool			 mainThreadOnly;
@@ -1745,6 +1970,8 @@ private:
                                     pjmedia_sdp_session *sdp,
                                     pj_pool_t *pool,
                                     const pjmedia_sdp_session *rem_sdp);
+    static void on_stream_precreate(pjsua_call_id call_id,
+                                    pjsua_on_stream_precreate_param *param);
     static void on_stream_created2(pjsua_call_id call_id,
 				   pjsua_on_stream_created_param *param);
     static void on_stream_destroyed(pjsua_call_id call_id,
@@ -1753,6 +1980,8 @@ private:
     static void on_dtmf_digit(pjsua_call_id call_id, int digit);
     static void on_dtmf_digit2(pjsua_call_id call_id, 
 			       const pjsua_dtmf_info *info);
+    static void on_dtmf_event(pjsua_call_id call_id,
+                              const pjsua_dtmf_event *event);
     static void on_call_transfer_request(pjsua_call_id call_id,
                                          const pj_str_t *dst,
                                          pjsip_status_code *code);
@@ -1816,6 +2045,15 @@ private:
 			  pj_status_t status,
 			  const pjsua_ip_change_op_info *info);
 
+    static pj_status_t on_auth_create_aka_response_callback(
+					     pj_pool_t *pool,
+					     const pjsip_digest_challenge*chal,
+					     const pjsip_cred_info *cred,
+					     const pj_str_t *method,
+					     pjsip_digest_credential *auth);
+    friend class Account;
+
+
 private:
     void clearCodecInfoList(CodecInfoVector &codec_list);
     void updateCodecInfoList(pjsua_codec_info pj_codec[], unsigned count,
