@@ -1,4 +1,4 @@
-/* $Id: siptypes.hpp 6026 2019-06-12 06:00:35Z nanang $ */
+/* $Id$ */
 /*
  * Copyright (C) 2032 Teluu Inc. (http://www.teluu.com)
  *
@@ -99,6 +99,16 @@ public:
                  const int data_type,
                  const string data);
 
+    /**
+     * Convert from pjsip
+     */
+    void fromPj(const pjsip_cred_info &prm);
+
+    /**
+     * Convert to pjsip
+     */
+    pjsip_cred_info toPj() const;
+
     /**
      * Read this object from a container node.
      *
@@ -315,6 +325,11 @@ struct TransportConfig : public PersistentObject
      * Specify the port range for socket binding, relative to the start
      * port number specified in \a port. Note that this setting is only
      * applicable when the start port number is non zero.
+     * 
+     * Example: \a port=5000, \a portRange=4
+     * - Available ports: 5000, 5001, 5002, 5003, 5004 (SIP transport)
+     * 
+     * Available ports are in the range of [\a port, \a port + \a portRange]. 
      *
      * Default value is zero.
      */
